﻿--[[
    Kodak Moments by Jezebelle @ Eitrigg (US)
    Copyright (C) 2011
]]--

local addonName = ...

local addonNiceName = "Kodak Moments"
local addonAuthor   = "Jezebelle @ Eitrigg [US]"
local addonVersion  = "v4.0.001"

_G[addonName]  = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")

local addon = _G[addonName]

local AddonDB_Defaults = {
    global = {
        options = {
            HideUI                  = false,
            Achievements            = true,
            Levels                  = true,
            Reputations             = true,
            Reputations_ExaltedOnly = true,
            BGs                     = true,
            BGs_WinsOnly            = true,
            Arenas                  = true,
            Arenas_WinsOnly         = true,
            Debug                   = false
        }
    }
}

function addon:OnInitialize()
    --Nothing to see here, move along!
    
end

function addon:OnEnable()
    --Load Options
    addon.db = LibStub("AceDB-3.0"):New(addonName .. "DB", AddonDB_Defaults, true)

    --Register events here
    addon:RegisterEvent("ACHIEVEMENT_EARNED")
    addon:RegisterEvent("PLAYER_LEVEL_UP")
    addon:RegisterEvent("CHAT_MSG_SYSTEM")
    addon:RegisterEvent("UPDATE_BATTLEFIELD_STATUS")

    addon:RegisterEvent("SCREENSHOT_SUCCEEDED", "PostScreenshot")
    addon:RegisterEvent("SCREENSHOT_FAILED", "PostScreenshot")
    
    addon:Hook("Screenshot", true)
    
    KodakMoments:Debug("Successfully enabled")
end

function addon:OnDisable()
    --Unregister events here
    addon:UnrgisterEvent("ACHIEVEMENT_EARNED")
    addon:UnregisterEvent("PLAYER_LEVEL_UP")
    addon:UnregisterEvent("CHAT_MSG_SYSTEM")
    addon:UnregisterEvent("UPDATE_BATTLEFIELD_STATUS")

    addon:UnregisterEvent("SCREENSHOT_SUCCEEDED", "PostScreenshot")
    addon:UnregisterEvent("SCREENSHOT_FAILED", "PostScreenshot")
    
    addon:UnhookAll()
end


function addon:ACHIEVEMENT_EARNED()
    addon:Debug("Achievement Earned")
    
    if ( addon:GetOption("Achievements") == true ) then
        addon:TakeScreenshot()
    end
end

function addon:PLAYER_LEVEL_UP(messageName, newLevel)
    addon:Debug("Leveled Gained (" .. newLevel .. ")")
    
    if ( addon:GetOption("Levels") == true ) then
        if ( addon:GetOption("Achievements") == true ) then
            --Major levels (10, 20... 80, 85) award achievements, so we can ignore these.
            if ( newLevel ~= 85 and newLevel%10 ~= 0 ) then
                addon:TakeScreenshot(1)
            end
        else
            addon:TakeScreenshot(1)
        end
    end
end

function addon:CHAT_MSG_SYSTEM(messageName, msg)
    addon:Debug("Chat Message Received")
    
    if ( addon:GetOption("Reputations") == true ) then
        if ( string.find(msg, "You are now") and string.find(msg, "with") ) then
            if ( addon:GetOption("Debug") == true ) then
                addon:Print("Chat Message Relates to Reputation Gain")
            end

            if ( addon:GetOption("Reputations_ExaltedOnly") == true ) then
                if ( string.find(msg, "Exalted") ) then
                    addon:TakeScreenshot()
                end
            else
                addon:TakeScreenshot()
            end
        end
    end
end

function addon:UPDATE_BATTLEFIELD_STATUS()
    addon:Debug("Battlefield Status Message Received")
    
    local winner = GetBattlefieldWinner()
    
    if( winner ~= nil ) then
        if ( IsActiveBattlefieldArena() == true ) then
            if ( addon:GetOption("Arenas") == true ) then
                if ( addon:GetOption("Arenas_WinsOnly") == true ) then
                    if ( winner == addon:GetPlayerArenaTeam() ) then
                        addon:TakeScreenshot()
                    end
                else
                    addon:TakeScreenshot()
                end
            end
        else
            if ( addon:GetOption("BGs") == true ) then
                if ( addon:GetOption("BGs_WinsOnly") == true ) then
                    if ( winner == addon:GetPlayerBGFaction() ) then
                        addon:TakeScreenshot()
                    end
                else
                    addon:TakeScreenshot()
                end
            end
        end
    end
end

--
--
function addon:GetPlayerBGFaction()
    addon:Debug("GetPlayerBGFaction()")
    
    if ( UnitFactionGroup("player") == "Horde" ) then
        return 0
    else
        return 1
    end
end

--
--
function addon:GetPlayerArenaTeam()
    addon:Debug("GetPlayerArenaTeam()")
    
    local greenTeamName = GetBattlefieldTeamInfo(0)
    
    for i = 1, 3 do
        if( greenTeamName == GetArenaTeam(i) ) then
            return 0
        end
    end

    return 1
end


--Hook and extend the default Screenshot() function to enable us to conditionally hide the UI
--
function addon:Screenshot(hideMe)
    if ( (addon:GetOption("HideUI") == true) and (hideMe == nil or hideMe == true) ) then
        UIParent:Hide()
    end
end

--Schedule the screenshot after a small delay
--Added a delay (default 0.5 seconds) to give the "sparkle" effects time to draw in
--
function addon:TakeScreenshot(delay)
    addon:Debug("TakeScreenshot()")
    
    if ( delay == nil ) then
        delay = 0.5
    end
    
    addon:ScheduleTimer("Snap", delay)
end

--Actually take the screenshot
--
function addon:Snap()
    Screenshot(false)
end

--After taking a screenshot, ensure that the UI is re-enabled
--
function addon:PostScreenshot()
    addon:Debug("PostScreenshot()")
    
    UIParent:Show()
end

--Get an option from the addon's global database by it's key
--
function addon:GetOption(key)
    if ( addon.db and addon.db.global ) then
        return addon.db.global.options[key]
    end
end

--Set an option in the addon's global database by it's key
--
function addon:SetOption(key, value)
    if ( addon.db and addon.db.global ) then
        addon.db.global.options[key] = value
    end
end

--Print debug messages, if enabled
--
function addon:Debug(msg)
    if ( addon:GetOption("Debug") == true ) then
        addon:Print("|cffff4040DEBUG:|r " .. msg)
    end
end